package com.chinaitop.depot.basic.controller;

import java.io.IOException;
import java.util.*;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.chinaitop.depot.basic.model.MonitorNvr;
import com.chinaitop.depot.basic.service.MonitorNvrService;
import com.hikvision.artemis.sdk.ArtemisHttpUtil;
import com.hikvision.artemis.sdk.config.ArtemisConfig;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.chinaitop.depot.basic.model.MonitorCamera;
import com.chinaitop.depot.basic.model.MonitorCameraExample;
import com.chinaitop.depot.basic.service.MonitorCameraService;
import com.chinaitop.depot.basic.service.impl.CameraStorehouseServiceImpl;
import com.chinaitop.depot.basic.service.impl.ImportServiceImpl;
import com.chinaitop.depot.utils.ParameterUtil;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.github.pagehelper.PageHelper;
import com.github.pagehelper.PageInfo;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiImplicitParam;
import io.swagger.annotations.ApiImplicitParams;
import io.swagger.annotations.ApiOperation;

/**
 * 摄像头操作类
 */
@RestController
@RequestMapping(value="/camera")
@Api(value= "MonitorCameraController", description = "摄像头操作类")
public class MonitorCameraController {

    @SuppressWarnings("unused")
	private static Logger log = LoggerFactory.getLogger(MonitorCameraController.class);

    @Resource
    private MonitorCameraService cameraService;

    @Resource
    private MonitorNvrService nvrService;
    
    @Resource
    private ImportServiceImpl importService;
    
    @Resource
    private CameraStorehouseServiceImpl cameraStorehouseService;

    @Value("${web.upload-path}")
    private String uploadPath;

    /**
     * 查找摄像头信息withPage
     * @param pageNum 页数
     * @param pageSize 每页条数
     * @param cameraName 摄像头名称
     * @param orgid 组织机构id
     * @return
     */
    @RequestMapping(value="/getCamera", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
	@ApiOperation(value="查找摄像头信息", notes = "查找摄像头信息，支持分页")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "pageNum", value = "页码", paramType = "query"),
		@ApiImplicitParam(name = "pageSize", value = "每页条数", paramType = "query"),
		@ApiImplicitParam(name = "cameraName", value = "摄像头名称", paramType = "query"),
        @ApiImplicitParam(name="orgid", value="组织机构id", paramType="query")
	})
    public PageInfo<MonitorCamera> getCamera(Integer pageNum, Integer pageSize,String cameraName,Integer orgid) {

        MonitorCameraExample example = new MonitorCameraExample();
        MonitorCameraExample.Criteria criteria = example.createCriteria();
        criteria.andIsremoveEqualTo("1");//默认加载未删除的数据

        if(ParameterUtil.isnotnull(cameraName)) {
            criteria.andAliasLike("%" + cameraName + "%");
        }

        criteria.andOrgIdEqualTo(orgid.toString());//添加组织

        if (pageNum!=null && pageSize!=null) {
            PageHelper.startPage(pageNum, pageSize);
        }

        List<MonitorCamera> list = cameraService.queryByExample(example);
        PageInfo<MonitorCamera> pageInfo = new PageInfo<MonitorCamera>(list);
        return pageInfo;
    }

    /**
     * 根据id查找摄像头信息
     * @param id 主键id
     * @return
     */
    @RequestMapping(value="/edit", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
	@ApiOperation(value="根据id查找摄像头信息", notes = "根据id查找摄像头信息")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "主键id", paramType = "query")
	})
    public Map<String, Object> edit(Integer id) {
        Map<String, Object> modelMap = new HashMap<String, Object>();

        if (id != null) {
            MonitorCamera camera = cameraService.findById(id);
            modelMap.put("cameraEdit",camera);

            MonitorNvr nvr = nvrService.loadDataById(Integer.parseInt(camera.getNvrId()));
            modelMap.put("nvrEdit",nvr);
        }
        return modelMap;
    }

    /**
     * 保存摄像头信息
     * @param cameraJson 摄像头数据
     * @param nvrId 硬盘录像机ID
     * @return
     * @throws Exception
     */
    @RequestMapping(value="/saveCamera", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
	@ApiOperation(value="保存摄像头信息", notes = "保存摄像头信息")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "cameraJson", value = "摄像头数据", paramType = "form"),
		@ApiImplicitParam(name = "nvrId", value = "硬盘录像机ID", paramType = "form")
	})
    public Map<String, Object> saveCamera(String cameraJson,String nvrId) throws Exception {
        Map<String, Object> modelMap = new HashMap<String, Object>();
        // JSON字符串转对象
        ObjectMapper mapper = new ObjectMapper();
        MonitorCamera camera = (MonitorCamera)mapper.readValue(cameraJson, MonitorCamera.class);
        camera.setIsremove("1");//默认设置删除标志
        camera.setUpdateTime(ParameterUtil.string2date(ParameterUtil.getSysDateTime()));//修改时间
        if (camera.getId() == null) {
            camera.setNvrId(nvrId);
            cameraService.add(camera);
        } else {
            cameraService.update(camera);
        }
        //先删除摄像头仓房管理信息
        cameraStorehouseService.deleteById(camera.getId());
        if(camera.getLive().equals("1")) {//只有仓内摄像头才关联仓房信息
        	//保存摄像头仓房管理信息
        	if(!camera.getStorehouseId().isEmpty()) {
        		cameraStorehouseService.add(camera);
        	}
        }
        
        modelMap.put("status", "success");
        return modelMap;
    }

    /**
     * 删除摄像头信息
     * @param id 主键id
     * @return
     * @throws IOException
     */
    @RequestMapping(value="/removeCamera", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.POST)
	@ApiOperation(value="删除摄像头信息", notes = "删除摄像头信息")
	@ApiImplicitParams({
		@ApiImplicitParam(name = "id", value = "主键id", paramType = "form")
	})
    public Map<String, Object> removeCamera(Integer id) throws IOException {
        Map<String, Object> modelMap = new HashMap<>();
        if (id != null) {
            cameraService.updateDeleteById(id);
            modelMap.put("status", "success");
        }
        return modelMap;
    }

    /**
     * 查找要录像的NVR信息
     * @param orgid 组织机构id
     * @return
     */
    @RequestMapping(value="/getNVRVideo", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
	@ApiOperation(value="查找要录像的NVR信息", notes = "查找要录像的NVR信息")
    @ApiImplicitParams({
        @ApiImplicitParam(name="orgid", value="组织机构id", paramType="query")
    })
    public List<Map<String,Object>> getNVRVideo(Integer orgid) {
        Map<String,Object> map = new HashMap<>();
        map.put("orgid",orgid.toString());
        List<Map<String,Object>> list = cameraService.selectByNVRDates(map);
        return list;
    }

    /**
     * 查找要代储库的NVR信息
     * @param orgid 组织机构id
     * @return
     */
    @RequestMapping(value="/getAgentVideo", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="查找要代储库的NVR信息", notes = "查找要代储库的NVR信息")
    @ApiImplicitParams({
            @ApiImplicitParam(name="orgid", value="组织机构id", paramType="query")
    })
    public List<List<Map<String,Object>>> getAgentVideo(Integer orgid) {
        Map<String,Object> map = new HashMap<>();
        map.put("nowOrgid",orgid.toString());
        //先查询出来粮库下边的所有代储库
        List<Map<String,Object>> depotList = cameraService.getAgentDepot(map);
        List<List<Map<String,Object>>> list = new ArrayList<List<Map<String,Object>>>();
        for (Map<String,Object> depot : depotList) {
            map.put("orgid",depot.get("depotId"));
            List<Map<String,Object>> cameraList = cameraService.getAgentVideo(map);
            if(cameraList!=null&&!cameraList.isEmpty()){
                if(depot.size() == 2){
                    cameraList = getCameraListURL(depot.get("hkptCode").toString(),cameraList);
                }
                list.add(cameraList);
            }
        }

        return list;
    }

    /**
     * 海康流媒体平台播放本库摄像头
     * @param orgid 组织机构id
     * @return
     */
    @RequestMapping(value="/getFlowPlayVideo", produces = MediaType.APPLICATION_JSON_VALUE, method = RequestMethod.GET)
    @ApiOperation(value="流媒体平台播放", notes = "海康流媒体平台播放本库摄像头")
    @ApiImplicitParams({
            @ApiImplicitParam(name="orgid", value="组织机构id", paramType="query")
    })
    public List<List<Map<String,Object>>> getFlowPlayVideo(Integer orgid) {
        Map<String,Object> map = new HashMap<>();
        map.put("orgid",orgid.toString());
        //先查询出来粮库下边的流媒体code码
//        String hkptCode = cameraService.getFlowPlayDepot(map);
        List<List<Map<String,Object>>> list = new ArrayList<List<Map<String,Object>>>();
        List<Map<String,Object>> cameraList = cameraService.selectByNVRDates(map);
        if(cameraList!=null&&!cameraList.isEmpty()){
            List<Map<String,Object>> KQList = getCameraListURL("6cfe8e38-c460-491b-a27d-ec7e6c9f9c0f",cameraList);
            if(KQList.size() > 0) {
                KQList.get(0).put("agentDepotName", "库区");
                list.add(KQList);
            }
            List<Map<String,Object>> YWList = new ArrayList<Map<String,Object>>();
            YWList = getCameraListURL("2e16f3f0-e92d-4abf-867f-2437b6768b9b",cameraList);
            if(YWList.size() > 0) {
                YWList.get(0).put("agentDepotName", "业务");
                list.add(YWList);
            }
            List<Map<String,Object>> AFList = new ArrayList<Map<String,Object>>();
            AFList = getCameraListURL("fc0d0679-9b73-4b35-b268-ef4038025827",cameraList);
            if(AFList.size() > 0) {
                AFList.get(0).put("agentDepotName", "安防");
                list.add(AFList);
            }
            List<Map<String,Object>> CNList = new ArrayList<Map<String,Object>>();
            CNList = getCameraListURL("17511dff-fee6-4242-ab55-ca9461592e5a",cameraList);
            if(CNList.size() > 0){
                CNList.get(0).put("agentDepotName","仓内");
                list.add(CNList);
            }
        }
        return list;
    }

    /**
     * 导入excel
     * @param file 文件
     * @param fileType 类型：如xlsx
     * @param orgId 组织机构id
     * @return
     */
    @RequestMapping(value="/importFile")
    public String importFile(MultipartFile file, String fileType, HttpServletRequest request,Integer orgId){

        String[] titleArr = {"摄像头名称", "摄像机别名", "所属厂商(200代表海康，300代表大华)", "账号", "密码", "硬盘录像机编号", "通道号", "IP地址",
        		"子网掩码", "网关", "设备类型(1代表球形摄像机,2代表枪式摄像机)", "设备端口", "设备型号", "所在位置(1代表仓内,2代表仓外)",
        		"设备序列号", "安装位置", "安装方式", "照射区域", "取电位置", "仓房名称"};//第一行的title
        boolean[] emptyArr = {false, false, false, false, false, false, false, false,
        		true, true, false, false, true, true,
        		true, true, true, true, true,true};//指定列可为空，不可为空值为false.

        List<List<String>> list = new ArrayList<List<String>>();//该list为数据返回的list
        String errormsg = null;
        try {

        	errormsg = importService.importFile(uploadPath, file, fileType, request, titleArr, emptyArr, list);

        	if (errormsg != null && !"".equals(errormsg)) {
        		return errormsg;
        	}

            if (list.size()>0) {
            	errormsg = cameraService.importData(list, orgId);

            	System.out.println(list);
            	System.out.println(errormsg);
            	//errormsg = "导入成功！";
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        return errormsg;
    }

    public List<Map<String,Object>> getCameraListURL(String regionIndexCode,List<Map<String,Object>> cameraList) {
        /**
         * STEP1：设置平台参数，根据实际情况,设置host appKey appSecret 三个参数.
         */
        // artemis网关服务器ip端口
        ArtemisConfig.host = "172.16.3.55:443";
        // 秘钥appKey
        ArtemisConfig.appKey = "24421703";
        // 秘钥appSecret
        ArtemisConfig.appSecret = "kwqBBk5io9oo970ueCQm";
        /**
         * STEP2：设置OpenAPI接口的上下文
         */
        final String ARTEMIS_PATH = "/artemis";
        String body = null;
        String contentType = null;
        Map  path = new HashMap();
        /**
         * STEP3：设置接口的URI地址
         * STEP4：设置参数提交方式 根据现场环境部署确认是http还是https
         * STEP5：组装请求参数
         */
        //获取区域列表
        final String previewURLsApi = ARTEMIS_PATH + "/api/resource/v1/regions/regionIndexCode/cameras";
        path = new HashMap (2) {{ put("https://", previewURLsApi); }};
        contentType = "application/json";
        JSONObject jsonBody = new JSONObject();
        jsonBody.put("pageNo", 1);
        jsonBody.put("pageSize", 100);
        jsonBody.put("regionIndexCode", regionIndexCode);
        body = jsonBody.toJSONString();
        /**
         * STEP6：调用接口   post请求application/json类型参数
         */
        String resultData = ArtemisHttpUtil.doPostStringArtemis(path, body, null, null, contentType, null);
        // 将String转化为json数据
        JSONObject objData = JSONObject.parseObject(resultData);
        JSONObject jsonObjData = objData.getJSONObject("data");
        JSONArray arrayList = JSON.parseArray(jsonObjData.getString("list"));
        Iterator<Object> it = arrayList.iterator();
        List<Map<String,Object>> dataList = new ArrayList<Map<String,Object>>();
        while (it.hasNext()) {
            JSONObject obj = (JSONObject) it.next();
            //监控点名称--判断监控名称是否一样，如果一样则赋值监控点唯一标识
            for (Map<String,Object> cameraData:cameraList) {
                if(ParameterUtil.isequal(cameraData.get("name"),obj.getString("cameraName"))){
                    //监控点唯一标识
                    cameraData.put("cameraIndexCode",obj.getString("cameraIndexCode"));
                    dataList.add(cameraData);
                };
            }
        }
        return dataList;
    }
}
